"""
SnapEngage template tags.
"""

from __future__ import absolute_import

import re

from django.conf import settings
from django.template import Library, Node, TemplateSyntaxError
from django.utils import translation

from analytical.utils import get_identity, get_required_setting

BUTTON_LOCATION_LEFT = 0
BUTTON_LOCATION_RIGHT = 1
BUTTON_LOCATION_TOP = 2
BUTTON_LOCATION_BOTTOM = 3

BUTTON_STYLE_NONE = 0
BUTTON_STYLE_DEFAULT = 1
BUTTON_STYLE_LIVE = 2

FORM_POSITION_TOP_LEFT = 'tl'
FORM_POSITION_TOP_RIGHT = 'tr'
FORM_POSITION_BOTTOM_LEFT = 'bl'
FORM_POSITION_BOTTOM_RIGHT = 'br'

WIDGET_ID_RE = re.compile(r'^[a-z0-9]{8}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{12}$')
SETUP_CODE = """
    <script type="text/javascript">
      document.write(unescape("%%3Cscript src='" + ((document.location.protocol=="https:")?"https://snapabug.appspot.com":"http://www.snapengage.com") + "/snapabug.js' type='text/javascript'%%3E%%3C/script%%3E"));</script><script type="text/javascript">
      %(settings_code)s
    </script>
"""  # noqa
DOMAIN_CODE = 'SnapABug.setDomain("%s");'
SECURE_CONNECTION_CODE = 'SnapABug.setSecureConnexion();'
INIT_CODE = 'SnapABug.init("%s");'
ADDBUTTON_CODE = 'SnapABug.addButton("%(id)s","%(location)s","%(offset)s"%(dynamic_tail)s);'
SETBUTTON_CODE = 'SnapABug.setButton("%s");'
SETEMAIL_CODE = 'SnapABug.setUserEmail("%s"%s);'
SETLOCALE_CODE = 'SnapABug.setLocale("%s");'
FORM_POSITION_CODE = 'SnapABug.setChatFormPosition("%s");'
FORM_TOP_POSITION_CODE = 'SnapABug.setFormTopPosition(%d);'
BUTTONEFFECT_CODE = 'SnapABug.setButtonEffect("%s");'
DISABLE_OFFLINE_CODE = 'SnapABug.allowOffline(false);'
DISABLE_SCREENSHOT_CODE = 'SnapABug.allowScreenshot(false);'
DISABLE_OFFLINE_SCREENSHOT_CODE = 'SnapABug.showScreenshotOption(false);'
DISABLE_PROACTIVE_CHAT_CODE = 'SnapABug.allowProactiveChat(false);'
DISABLE_SOUNDS_CODE = 'SnapABug.allowChatSound(false);'

register = Library()


@register.tag
def snapengage(parser, token):
    """
    SnapEngage set-up template tag.

    Renders Javascript code to set-up SnapEngage chat.  You must supply
    your widget ID in the ``SNAPENGAGE_WIDGET_ID`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return SnapEngageNode()


class SnapEngageNode(Node):
    def __init__(self):
        self.widget_id = get_required_setting(
            'SNAPENGAGE_WIDGET_ID', WIDGET_ID_RE,
            "must be a string looking like this: 'XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX'")

    def render(self, context):
        settings_code = []

        domain = self._get_setting(context, 'snapengage_domain',
                                   'SNAPENGAGE_DOMAIN')
        if domain is not None:
            settings_code.append(DOMAIN_CODE % domain)

        secure_connection = self._get_setting(context,
                                              'snapengage_secure_connection',
                                              'SNAPENGAGE_SECURE_CONNECTION',
                                              False)
        if secure_connection:
            settings_code.append(SECURE_CONNECTION_CODE)

        email = context.get('snapengage_email')
        if email is None:
            email = get_identity(context, 'snapengage', lambda u: u.email)
        if email is not None:
            if self._get_setting(context, 'snapengage_readonly_email',
                                 'SNAPENGAGE_READONLY_EMAIL', False):
                readonly_tail = ',true'
            else:
                readonly_tail = ''
            settings_code.append(SETEMAIL_CODE % (email, readonly_tail))

        locale = self._get_setting(context, 'snapengage_locale',
                                   'SNAPENGAGE_LOCALE')
        if locale is None:
            locale = translation.to_locale(translation.get_language())
        settings_code.append(SETLOCALE_CODE % locale)

        form_position = self._get_setting(context,
                                          'snapengage_form_position', 'SNAPENGAGE_FORM_POSITION')
        if form_position is not None:
            settings_code.append(FORM_POSITION_CODE % form_position)

        form_top_position = self._get_setting(context,
                                              'snapengage_form_top_position',
                                              'SNAPENGAGE_FORM_TOP_POSITION')
        if form_top_position is not None:
            settings_code.append(FORM_TOP_POSITION_CODE % form_top_position)

        show_offline = self._get_setting(context, 'snapengage_show_offline',
                                         'SNAPENGAGE_SHOW_OFFLINE', True)
        if not show_offline:
            settings_code.append(DISABLE_OFFLINE_CODE)

        screenshots = self._get_setting(context, 'snapengage_screenshots',
                                        'SNAPENGAGE_SCREENSHOTS', True)
        if not screenshots:
            settings_code.append(DISABLE_SCREENSHOT_CODE)

        offline_screenshots = self._get_setting(context,
                                                'snapengage_offline_screenshots',
                                                'SNAPENGAGE_OFFLINE_SCREENSHOTS', True)
        if not offline_screenshots:
            settings_code.append(DISABLE_OFFLINE_SCREENSHOT_CODE)

        if not context.get('snapengage_proactive_chat', True):
            settings_code.append(DISABLE_PROACTIVE_CHAT_CODE)

        sounds = self._get_setting(context, 'snapengage_sounds',
                                   'SNAPENGAGE_SOUNDS', True)
        if not sounds:
            settings_code.append(DISABLE_SOUNDS_CODE)

        button_effect = self._get_setting(context, 'snapengage_button_effect',
                                          'SNAPENGAGE_BUTTON_EFFECT')
        if button_effect is not None:
            settings_code.append(BUTTONEFFECT_CODE % button_effect)

        button = self._get_setting(context, 'snapengage_button',
                                   'SNAPENGAGE_BUTTON', BUTTON_STYLE_DEFAULT)
        if button == BUTTON_STYLE_NONE:
            settings_code.append(INIT_CODE % self.widget_id)
        else:
            if not isinstance(button, int):
                # Assume button as a URL to a custom image
                settings_code.append(SETBUTTON_CODE % button)
            button_location = self._get_setting(
                context,
                'snapengage_button_location', 'SNAPENGAGE_BUTTON_LOCATION',
                BUTTON_LOCATION_LEFT)
            button_offset = self._get_setting(
                context,
                'snapengage_button_location_offset',
                'SNAPENGAGE_BUTTON_LOCATION_OFFSET', '55%')
            settings_code.append(ADDBUTTON_CODE % {
                'id': self.widget_id,
                'location': button_location,
                'offset': button_offset,
                'dynamic_tail': ',true' if (button == BUTTON_STYLE_LIVE) else '',
            })
        html = SETUP_CODE % {
            'widget_id': self.widget_id,
            'settings_code': " ".join(settings_code),
        }
        return html

    def _get_setting(self, context, context_key, setting=None, default=None):
        try:
            return context[context_key]
        except KeyError:
            if setting is not None:
                return getattr(settings, setting, default)
            else:
                return default


def contribute_to_analytical(add_node):
    SnapEngageNode()  # ensure properly configured
    add_node('body_bottom', SnapEngageNode)



"""
django-analytical testing settings.
"""

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django.contrib.sites',
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'analytical',
]

SECRET_KEY = 'testing'

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
    },
]



"""
Tests for the Google Analytics template tags and filters, using the new gtag.js library.
"""

from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.google_analytics_gtag import GoogleAnalyticsGTagNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(GOOGLE_ANALYTICS_GTAG_PROPERTY_ID='UA-123456-7')
class GoogleAnalyticsTagTestCase(TagTestCase):
    """
    Tests for the ``google_analytics_js`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('google_analytics_gtag', 'google_analytics_gtag')
        self.assertTrue(
            '<script async src="https://www.googletagmanager.com/gtag/js?id=UA-123456-7"></script>'
            in r, r)
        self.assertTrue("gtag('js', new Date());" in r, r)
        self.assertTrue("gtag('config', 'UA-123456-7');" in r, r)

    def test_node(self):
        r = GoogleAnalyticsGTagNode().render(Context())
        self.assertTrue(
            '<script async src="https://www.googletagmanager.com/gtag/js?id=UA-123456-7"></script>'
            in r, r)
        self.assertTrue("gtag('js', new Date());" in r, r)
        self.assertTrue("gtag('config', 'UA-123456-7');" in r, r)

    @override_settings(GOOGLE_ANALYTICS_GTAG_PROPERTY_ID=None)
    def test_no_property_id(self):
        self.assertRaises(AnalyticalException, GoogleAnalyticsGTagNode)

    @override_settings(GOOGLE_ANALYTICS_GTAG_PROPERTY_ID='wrong')
    def test_wrong_property_id(self):
        self.assertRaises(AnalyticalException, GoogleAnalyticsGTagNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = GoogleAnalyticsGTagNode().render(context)
        self.assertTrue(r.startswith(
            '<!-- Google Analytics disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Tests for the KISSmetrics tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.kiss_metrics import KissMetricsNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(KISS_METRICS_API_KEY='0123456789abcdef0123456789abcdef01234567')
class KissMetricsTagTestCase(TagTestCase):
    """
    Tests for the ``kiss_metrics`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('kiss_metrics', 'kiss_metrics')
        self.assertTrue("//doug1izaerwt3.cloudfront.net/"
                        "0123456789abcdef0123456789abcdef01234567.1.js" in r, r)

    def test_node(self):
        r = KissMetricsNode().render(Context())
        self.assertTrue("//doug1izaerwt3.cloudfront.net/"
                        "0123456789abcdef0123456789abcdef01234567.1.js" in r, r)

    @override_settings(KISS_METRICS_API_KEY=None)
    def test_no_api_key(self):
        self.assertRaises(AnalyticalException, KissMetricsNode)

    @override_settings(KISS_METRICS_API_KEY='0123456789abcdef0123456789abcdef0123456')
    def test_api_key_too_short(self):
        self.assertRaises(AnalyticalException, KissMetricsNode)

    @override_settings(KISS_METRICS_API_KEY='0123456789abcdef0123456789abcdef012345678')
    def test_api_key_too_long(self):
        self.assertRaises(AnalyticalException, KissMetricsNode)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = KissMetricsNode().render(Context({'user': User(username='test')}))
        self.assertTrue("_kmq.push(['identify', 'test']);" in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = KissMetricsNode().render(Context({'user': AnonymousUser()}))
        self.assertFalse("_kmq.push(['identify', " in r, r)

    def test_event(self):
        r = KissMetricsNode().render(Context({
            'kiss_metrics_event': ('test_event', {'prop1': 'val1', 'prop2': 'val2'}),
        }))
        self.assertTrue("_kmq.push(['record', 'test_event', "
                        '{"prop1": "val1", "prop2": "val2"}]);' in r, r)

    def test_property(self):
        r = KissMetricsNode().render(Context({
            'kiss_metrics_properties': {'prop1': 'val1', 'prop2': 'val2'},
        }))
        self.assertTrue("_kmq.push([\'set\', "
                        '{"prop1": "val1", "prop2": "val2"}]);' in r, r)

    def test_alias(self):
        r = KissMetricsNode().render(Context({
            'kiss_metrics_alias': {'test': 'test_alias'},
        }))
        self.assertTrue("_kmq.push(['alias', 'test', 'test_alias']);" in r, r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = KissMetricsNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- KISSmetrics disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Crazy Egg template tags and filters.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import is_internal_ip, disable_html, get_required_setting


ACCOUNT_NUMBER_RE = re.compile(r'^\d+$')
SETUP_CODE = '<script type="text/javascript" src="{placeholder_url}">' \
             '</script>'.\
    format(placeholder_url='//dnn506yrbagrg.cloudfront.net/pages/scripts/'
                           '%(account_nr_1)s/%(account_nr_2)s.js')
USERVAR_CODE = "CE2.set(%(varnr)d, '%(value)s');"


register = Library()


@register.tag
def crazy_egg(parser, token):
    """
    Crazy Egg tracking template tag.

    Renders Javascript code to track page clicks.  You must supply
    your Crazy Egg account number (as a string) in the
    ``CRAZY_EGG_ACCOUNT_NUMBER`` setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return CrazyEggNode()


class CrazyEggNode(Node):
    def __init__(self):
        self.account_nr = get_required_setting(
            'CRAZY_EGG_ACCOUNT_NUMBER',
            ACCOUNT_NUMBER_RE, "must be (a string containing) a number"
        )

    def render(self, context):
        html = SETUP_CODE % {
            'account_nr_1': self.account_nr[:4],
            'account_nr_2': self.account_nr[4:],
        }
        values = (context.get('crazy_egg_var%d' % i) for i in range(1, 6))
        params = [(i, v) for i, v in enumerate(values, 1) if v is not None]
        if params:
            js = " ".join(USERVAR_CODE % {
                'varnr': varnr,
                'value': value,
            } for (varnr, value) in params)
            html = '%s\n' \
                   '<script type="text/javascript">%s</script>' % (html, js)
        if is_internal_ip(context, 'CRAZY_EGG'):
            html = disable_html(html, 'Crazy Egg')
        return html


def contribute_to_analytical(add_node):
    CrazyEggNode()  # ensure properly configured
    add_node('body_bottom', CrazyEggNode)



import os

try:
    from setuptools import setup, Command
except ImportError:
    from distutils.core import setup, Command

os.environ['DJANGO_SETTINGS_MODULE'] = 'analytical.tests.settings'

cmdclass = {}

try:
    from sphinx.setup_command import BuildDoc

    cmdclass['build_sphinx'] = BuildDoc
except ImportError:
    pass


class TestCommand(Command):
    description = "run package tests"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from analytical.tests.utils import run_tests

        run_tests()


cmdclass['test'] = TestCommand


def read_file(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


try:
    import django
    os.environ.setdefault(
        "DJANGO_SETTINGS_MODULE",
        "analytical.tests.settings"
    )
    django.setup()
except ImportError:
    print(
        "Could not import django. "
        "This is fine, unless you intend to run unit tests."
    )

import analytical as package  # noqa

setup(
    name='django-analytical',
    version=package.__version__,
    license=package.__license__,
    description=package.__doc__.strip(),
    long_description=read_file('README.rst'),
    long_description_content_type='text/x-rst',
    author=package.__author__,
    author_email=package.__email__,
    packages=[
        'analytical',
        'analytical.templatetags',
        'analytical.tests',
        'analytical.tests.templatetags',
    ],
    keywords=['django', 'analytics'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    platforms=['any'],
    url='https://github.com/jazzband/django-analytical',
    download_url='https://github.com/jazzband/django-analytical/archive/master.zip',
    cmdclass=cmdclass,
)



"""
Tests for the Rating@Mail.ru template tags and filters.
"""

from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.rating_mailru import RatingMailruNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(RATING_MAILRU_COUNTER_ID='1234567')
class RatingMailruTagTestCase(TagTestCase):
    """
    Tests for the ``rating_mailru`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('rating_mailru', 'rating_mailru')
        self.assertTrue("counter?id=1234567;js=na" in r, r)

    def test_node(self):
        r = RatingMailruNode().render(Context({}))
        self.assertTrue("counter?id=1234567;js=na" in r, r)

    @override_settings(RATING_MAILRU_COUNTER_ID=None)
    def test_no_site_id(self):
        self.assertRaises(AnalyticalException, RatingMailruNode)

    @override_settings(RATING_MAILRU_COUNTER_ID='1234abc')
    def test_wrong_site_id(self):
        self.assertRaises(AnalyticalException, RatingMailruNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = RatingMailruNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Rating@Mail.ru disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Facebook Pixel template tags and filters.
"""
from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import get_required_setting, is_internal_ip, disable_html


FACEBOOK_PIXEL_HEAD_CODE = """\
<script>
  !function(f,b,e,v,n,t,s)
  {if(f.fbq)return;n=f.fbq=function(){n.callMethod?
  n.callMethod.apply(n,arguments):n.queue.push(arguments)};
  if(!f._fbq)f._fbq=n;n.push=n;n.loaded=!0;n.version='2.0';
  n.queue=[];t=b.createElement(e);t.async=!0;
  t.src=v;s=b.getElementsByTagName(e)[0];
  s.parentNode.insertBefore(t,s)}(window, document,'script',
  'https://connect.facebook.net/en_US/fbevents.js');
  fbq('init', '%(FACEBOOK_PIXEL_ID)s');
  fbq('track', 'PageView');
</script>
"""

FACEBOOK_PIXEL_BODY_CODE = """\
<noscript><img height="1" width="1" style="display:none"
  src="https://www.facebook.com/tr?id=%(FACEBOOK_PIXEL_ID)s&ev=PageView&noscript=1"
/></noscript>
"""

register = Library()


def _validate_no_args(token):
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])


@register.tag
def facebook_pixel_head(parser, token):
    """
    Facebook Pixel head template tag.
    """
    _validate_no_args(token)
    return FacebookPixelHeadNode()


@register.tag
def facebook_pixel_body(parser, token):
    """
    Facebook Pixel body template tag.
    """
    _validate_no_args(token)
    return FacebookPixelBodyNode()


class _FacebookPixelNode(Node):
    """
    Base class: override and provide code_template.
    """
    def __init__(self):
        self.pixel_id = get_required_setting(
            'FACEBOOK_PIXEL_ID',
            re.compile(r'^\d+$'),
            "must be (a string containing) a number",
        )

    def render(self, context):
        html = self.code_template % {'FACEBOOK_PIXEL_ID': self.pixel_id}
        if is_internal_ip(context, 'FACEBOOK_PIXEL'):
            return disable_html(html, 'Facebook Pixel')
        else:
            return html

    @property
    def code_template(self):
        raise NotImplementedError  # pragma: no cover


class FacebookPixelHeadNode(_FacebookPixelNode):
    code_template = FACEBOOK_PIXEL_HEAD_CODE


class FacebookPixelBodyNode(_FacebookPixelNode):
    code_template = FACEBOOK_PIXEL_BODY_CODE


def contribute_to_analytical(add_node):
    # ensure properly configured
    FacebookPixelHeadNode()
    FacebookPixelBodyNode()
    add_node('head_bottom', FacebookPixelHeadNode)
    add_node('body_bottom', FacebookPixelBodyNode)



"""
Clicky template tags and filters.
"""

from __future__ import absolute_import

import json
import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import get_identity, is_internal_ip, disable_html, \
        get_required_setting


SITE_ID_RE = re.compile(r'^\d+$')
TRACKING_CODE = """
    <script type="text/javascript">
    var clicky = { log: function(){ return; }, goal: function(){ return; }};
    var clicky_site_ids = clicky_site_ids || [];
    clicky_site_ids.push(%(site_id)s);
    var clicky_custom = %(custom)s;
    (function() {
      var s = document.createElement('script');
      s.type = 'text/javascript';
      s.async = true;
      s.src = '//static.getclicky.com/js';
      ( document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0] ).appendChild( s );
    })();
    </script>
    <noscript><p><img alt="Clicky" width="1" height="1" src="//in.getclicky.com/%(site_id)sns.gif" /></p></noscript>
"""  # noqa

register = Library()


@register.tag
def clicky(parser, token):
    """
    Clicky tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your Clicky Site ID (as a string) in the ``CLICKY_SITE_ID``
    setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return ClickyNode()


class ClickyNode(Node):
    def __init__(self):
        self.site_id = get_required_setting(
            'CLICKY_SITE_ID', SITE_ID_RE,
            "must be a (string containing) a number")

    def render(self, context):
        custom = {}
        for dict_ in context:
            for var, val in dict_.items():
                if var.startswith('clicky_'):
                    custom[var[7:]] = val
        if 'username' not in custom.get('session', {}):
            identity = get_identity(context, 'clicky')
            if identity is not None:
                custom.setdefault('session', {})['username'] = identity

        html = TRACKING_CODE % {
            'site_id': self.site_id,
            'custom': json.dumps(custom, sort_keys=True),
        }
        if is_internal_ip(context, 'CLICKY'):
            html = disable_html(html, 'Clicky')
        return html


def contribute_to_analytical(add_node):
    ClickyNode()  # ensure properly configured
    add_node('body_bottom', ClickyNode)



"""
Gaug.es template tags and filters.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import is_internal_ip, disable_html, get_required_setting

SITE_ID_RE = re.compile(r'[\da-f]+$')
TRACKING_CODE = """
    <script type="text/javascript">
      var _gauges = _gauges || [];
      (function() {
        var t   = document.createElement('script');
        t.type  = 'text/javascript';
        t.async = true;
        t.id    = 'gauges-tracker';
        t.setAttribute('data-site-id', '%(site_id)s');
        t.src = '//secure.gaug.es/track.js';
        var s = document.getElementsByTagName('script')[0];
        s.parentNode.insertBefore(t, s);
      })();
    </script>
"""

register = Library()


@register.tag
def gauges(parser, token):
    """
    Gaug.es template tag.

    Renders Javascript code to gaug.es testing.  You must supply
    your Site ID account number in the ``GAUGES_SITE_ID``
    setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return GaugesNode()


class GaugesNode(Node):
    def __init__(self):
        self.site_id = get_required_setting(
                'GAUGES_SITE_ID', SITE_ID_RE,
                "must be a string looking like 'XXXXXXX'")

    def render(self, context):
        html = TRACKING_CODE % {'site_id': self.site_id}
        if is_internal_ip(context, 'GAUGES'):
            html = disable_html(html, 'Gauges')
        return html


def contribute_to_analytical(add_node):
    GaugesNode()
    add_node('head_bottom', GaugesNode)



"""
Tests for the intercom template tags and filters.
"""

import datetime

from django.contrib.auth.models import User
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.intercom import IntercomNode, intercom_user_hash, _timestamp
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(INTERCOM_APP_ID="abc123xyz")
class IntercomTagTestCase(TagTestCase):
    """
    Tests for the ``intercom`` template tag.
    """

    def test_tag(self):
        rendered_tag = self.render_tag('intercom', 'intercom')
        self.assertTrue(rendered_tag.strip().startswith('<script id="IntercomSettingsScriptTag">'))

    def test_node(self):
        now = datetime.datetime(2014, 4, 9, 15, 15, 0)
        user = User.objects.create(
            username='test',
            first_name='Firstname',
            last_name='Lastname',
            email="test@example.com",
            date_joined=now,
        )
        rendered_tag = IntercomNode().render(Context({'user': user}))
        # Because the json isn't predictably ordered, we can't just test the whole thing verbatim.
        self.assertEqual("""
<script id="IntercomSettingsScriptTag">
  window.intercomSettings = {"app_id": "abc123xyz", "created_at": 1397074500, "email": "test@example.com", "name": "Firstname Lastname", "user_id": %(user_id)s};
</script>
<script>(function(){var w=window;var ic=w.Intercom;if(typeof ic==="function"){ic('reattach_activator');ic('update',intercomSettings);}else{var d=document;var i=function(){i.c(arguments)};i.q=[];i.c=function(args){i.q.push(args)};w.Intercom=i;function l(){var s=d.createElement('script');s.type='text/javascript';s.async=true;s.src='https://static.intercomcdn.com/intercom.v1.js';var x=d.getElementsByTagName('script')[0];x.parentNode.insertBefore(s,x);}if(w.attachEvent){w.attachEvent('onload',l);}else{w.addEventListener('load',l,false);}}})()</script>
""" % {'user_id': user.pk}, rendered_tag)  # noqa

    @override_settings(INTERCOM_APP_ID=None)
    def test_no_account_number(self):
        self.assertRaises(AnalyticalException, IntercomNode)

    @override_settings(INTERCOM_APP_ID='123abQ')
    def test_wrong_account_number(self):
        self.assertRaises(AnalyticalException, IntercomNode)

    def test_identify_name_email_and_created_at(self):
        now = datetime.datetime(2014, 4, 9, 15, 15, 0)
        user = User.objects.create(
            username='test',
            first_name='Firstname',
            last_name='Lastname',
            email="test@example.com",
            date_joined=now,
        )
        r = IntercomNode().render(Context({
            'user': user,
        }))
        self.assertTrue('window.intercomSettings = {'
                        '"app_id": "abc123xyz", "created_at": 1397074500, '
                        '"email": "test@example.com", "name": "Firstname Lastname", '
                        '"user_id": %(user_id)s'
                        '};' % {'user_id': user.pk} in r, msg=r)

    def test_custom(self):
        r = IntercomNode().render(Context({
                'intercom_var1': 'val1',
                'intercom_var2': 'val2'
        }))
        self.assertTrue('var1": "val1", "var2": "val2"' in r)

    def test_identify_name_and_email(self):
        r = IntercomNode().render(Context({
                'user': User(
                    username='test',
                    first_name='Firstname',
                    last_name='Lastname',
                    email="test@example.com"),
        }))
        self.assertTrue('"email": "test@example.com", "name": "Firstname Lastname"' in r)

    def test_identify_username_no_email(self):
        r = IntercomNode().render(Context({'user': User(username='test')}))
        self.assertTrue('"name": "test"' in r, r)

    def test_no_identify_when_explicit_name(self):
        r = IntercomNode().render(Context({
            'intercom_name': 'explicit',
            'user': User(username='implicit'),
        }))
        self.assertTrue('"name": "explicit"' in r, r)

    def test_no_identify_when_explicit_email(self):
        r = IntercomNode().render(Context({
            'intercom_email': 'explicit',
            'user': User(username='implicit'),
        }))
        self.assertTrue('"email": "explicit"' in r, r)

    @override_settings(INTERCOM_HMAC_SECRET_KEY='secret')
    def test_user_hash__without_user_details(self):
        """
        No `user_hash` without `user_id` or `email`.
        """
        attrs = IntercomNode()._get_custom_attrs(Context())
        self.assertEqual({
            'created_at': None,
        }, attrs)

    @override_settings(INTERCOM_HMAC_SECRET_KEY='secret')
    def test_user_hash__with_user(self):
        """
        'user_hash' of default `user_id`.
        """
        user = User.objects.create(
            email='test@example.com',
        )  # type: User
        attrs = IntercomNode()._get_custom_attrs(Context({'user': user}))
        self.assertEqual({
            'created_at': int(_timestamp(user.date_joined)),
            'email': 'test@example.com',
            'name': '',
            'user_hash': intercom_user_hash(str(user.pk)),
            'user_id': user.pk,
        }, attrs)

    @override_settings(INTERCOM_HMAC_SECRET_KEY='secret')
    def test_user_hash__with_explicit_user_id(self):
        """
        'user_hash' of context-provided `user_id`.
        """
        attrs = IntercomNode()._get_custom_attrs(Context({
            'intercom_email': 'test@example.com',
            'intercom_user_id': '5',
        }))
        self.assertEqual({
            'created_at': None,
            'email': 'test@example.com',
            # HMAC for user_id:
            'user_hash': 'd3123a7052b42272d9b520235008c248a5aff3221cc0c530b754702ad91ab102',
            'user_id': '5',
        }, attrs)

    @override_settings(INTERCOM_HMAC_SECRET_KEY='secret')
    def test_user_hash__with_explicit_email(self):
        """
        'user_hash' of context-provided `email`.
        """
        attrs = IntercomNode()._get_custom_attrs(Context({
            'intercom_email': 'test@example.com',
        }))
        self.assertEqual({
            'created_at': None,
            'email': 'test@example.com',
            # HMAC for email:
            'user_hash': '49e43229ee99dca2565241719b8341b04e71dd4de0628f991b5bea30a526e153',
        }, attrs)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = IntercomNode().render(context)
        self.assertTrue(r.startswith('<!-- Intercom disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Tests for the Chartbeat template tags and filters.
"""

import re

from django.http import HttpRequest
from django.template import Context
from django.test import TestCase
from django.test.utils import override_settings

from analytical.templatetags.chartbeat import ChartbeatTopNode, \
        ChartbeatBottomNode
from analytical.tests.utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(CHARTBEAT_USER_ID='12345')
class ChartbeatTagTestCaseNoSites(TestCase):
    def test_rendering_setup_no_site(self):
        r = ChartbeatBottomNode().render(Context())
        self.assertTrue('var _sf_async_config={"uid": "12345"};' in r, r)


@override_settings(INSTALLED_APPS=(
    'analytical',
    'django.contrib.sites',
    'django.contrib.auth',
    'django.contrib.contenttypes',
))
@override_settings(CHARTBEAT_USER_ID='12345')
class ChartbeatTagTestCaseWithSites(TestCase):
    def setUp(self):
        from django.core.management import call_command
        call_command("migrate", verbosity=0)

    def test_rendering_setup_site(self):
        from django.contrib.sites.models import Site
        site = Site.objects.create(domain="test.com", name="test")
        with override_settings(SITE_ID=site.id):
            r = ChartbeatBottomNode().render(Context())
            self.assertTrue(re.search('var _sf_async_config={.*"uid": "12345".*};', r), r)
            self.assertTrue(re.search('var _sf_async_config={.*"domain": "test.com".*};', r), r)

    @override_settings(CHARTBEAT_AUTO_DOMAIN=False)
    def test_auto_domain_false(self):
        """
        Even if 'django.contrib.sites' is in INSTALLED_APPS, if
        CHARTBEAT_AUTO_DOMAIN is False, ensure there is no 'domain'
        in _sf_async_config.
        """
        r = ChartbeatBottomNode().render(Context())
        self.assertTrue('var _sf_async_config={"uid": "12345"};' in r, r)


@override_settings(CHARTBEAT_USER_ID='12345')
class ChartbeatTagTestCase(TagTestCase):
    """
    Tests for the ``chartbeat`` template tag.
    """

    def test_top_tag(self):
        r = self.render_tag('chartbeat', 'chartbeat_top', {'chartbeat_domain': "test.com"})
        self.assertTrue('var _sf_startpt=(new Date()).getTime()' in r, r)

    def test_bottom_tag(self):
        r = self.render_tag('chartbeat', 'chartbeat_bottom', {'chartbeat_domain': "test.com"})
        self.assertTrue(re.search('var _sf_async_config={.*"uid": "12345".*};', r), r)
        self.assertTrue(re.search('var _sf_async_config={.*"domain": "test.com".*};', r), r)

    def test_top_node(self):
        r = ChartbeatTopNode().render(Context({
            'chartbeat_domain': "test.com",
        }))
        self.assertTrue('var _sf_startpt=(new Date()).getTime()' in r, r)

    def test_bottom_node(self):
        r = ChartbeatBottomNode().render(Context({
            'chartbeat_domain': "test.com",
        }))
        self.assertTrue(re.search('var _sf_async_config={.*"uid": "12345".*};', r), r)
        self.assertTrue(re.search('var _sf_async_config={.*"domain": "test.com".*};', r), r)

    @override_settings(CHARTBEAT_USER_ID=None)
    def test_no_user_id(self):
        self.assertRaises(AnalyticalException, ChartbeatBottomNode)

    @override_settings(CHARTBEAT_USER_ID='123abc')
    def test_wrong_user_id(self):
        self.assertRaises(AnalyticalException, ChartbeatBottomNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = ChartbeatBottomNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Chartbeat disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Chartbeat template tags and filters.
"""

from __future__ import absolute_import

import json
import re

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import is_internal_ip, disable_html, get_required_setting


USER_ID_RE = re.compile(r'^\d+$')
INIT_CODE = """<script type="text/javascript">var _sf_startpt=(new Date()).getTime()</script>"""
SETUP_CODE = """
    <script type="text/javascript">
      var _sf_async_config=%(config)s;
      (function(){
        function loadChartbeat() {
          window._sf_endpt=(new Date()).getTime();
          var e = document.createElement('script');
          e.setAttribute('language', 'javascript');
          e.setAttribute('type', 'text/javascript');
          e.setAttribute('src',
             (("https:" == document.location.protocol) ? "https://a248.e.akamai.net/chartbeat.download.akamai.com/102508/" : "http://static.chartbeat.com/") +
             "js/chartbeat.js");
          document.body.appendChild(e);
        }
        var oldonload = window.onload;
        window.onload = (typeof window.onload != 'function') ?
           loadChartbeat : function() { oldonload(); loadChartbeat(); };
      })();
    </script>
"""  # noqa
DOMAIN_CONTEXT_KEY = 'chartbeat_domain'


register = Library()


@register.tag
def chartbeat_top(parser, token):
    """
    Top Chartbeat template tag.

    Render the top Javascript code for Chartbeat.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return ChartbeatTopNode()


class ChartbeatTopNode(Node):
    def render(self, context):
        if is_internal_ip(context):
            return disable_html(INIT_CODE, "Chartbeat")
        return INIT_CODE


@register.tag
def chartbeat_bottom(parser, token):
    """
    Bottom Chartbeat template tag.

    Render the bottom Javascript code for Chartbeat.  You must supply
    your Chartbeat User ID (as a string) in the ``CHARTBEAT_USER_ID``
    setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return ChartbeatBottomNode()


class ChartbeatBottomNode(Node):
    def __init__(self):
        self.user_id = get_required_setting('CHARTBEAT_USER_ID', USER_ID_RE,
                                            "must be (a string containing) a number")

    def render(self, context):
        config = {'uid': self.user_id}
        domain = _get_domain(context)
        if domain is not None:
            config['domain'] = domain
        html = SETUP_CODE % {'config': json.dumps(config, sort_keys=True)}
        if is_internal_ip(context, 'CHARTBEAT'):
            html = disable_html(html, 'Chartbeat')
        return html


def contribute_to_analytical(add_node):
    ChartbeatBottomNode()  # ensure properly configured
    add_node('head_top', ChartbeatTopNode, 'first')
    add_node('body_bottom', ChartbeatBottomNode, 'last')


def _get_domain(context):
    domain = context.get(DOMAIN_CONTEXT_KEY)

    if domain is not None:
        return domain
    else:
        if 'django.contrib.sites' not in settings.INSTALLED_APPS:
            return
        elif getattr(settings, 'CHARTBEAT_AUTO_DOMAIN', True):
            from django.contrib.sites.models import Site
            try:
                return Site.objects.get_current().domain
            except (ImproperlyConfigured, Site.DoesNotExist):  # pylint: disable=E1101
                return



"""
UserVoice template tags.
"""

from __future__ import absolute_import

import json
import re

from django.conf import settings
from django.template import Library, Node, TemplateSyntaxError
from analytical.utils import get_required_setting, get_identity


WIDGET_KEY_RE = re.compile(r'^[a-zA-Z0-9]*$')
TRACKING_CODE = """
    <script type="text/javascript">

    UserVoice=window.UserVoice||[];(function(){
            var uv=document.createElement('script');uv.type='text/javascript';
            uv.async=true;uv.src='//widget.uservoice.com/%(widget_key)s.js';
            var s=document.getElementsByTagName('script')[0];
            s.parentNode.insertBefore(uv,s)})();

    UserVoice.push(['set', %(options)s]);
    %(trigger)s
    %(identity)s
    </script>
"""
IDENTITY = """UserVoice.push(['identify', %(options)s]);"""
TRIGGER = "UserVoice.push(['addTrigger', {}]);"
register = Library()


@register.tag
def uservoice(parser, token):
    """
    UserVoice tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your UserVoice Widget Key in the ``USERVOICE_WIDGET_KEY``
    setting or the ``uservoice_widget_key`` template context variable.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return UserVoiceNode()


class UserVoiceNode(Node):
    def __init__(self):
        self.default_widget_key = get_required_setting(
            'USERVOICE_WIDGET_KEY', WIDGET_KEY_RE, "must be an alphanumeric string")

    def render(self, context):
        widget_key = context.get('uservoice_widget_key')
        if not widget_key:
            widget_key = self.default_widget_key
        if not widget_key:
            return ''
        # default
        options = {}
        options.update(getattr(settings, 'USERVOICE_WIDGET_OPTIONS', {}))
        options.update(context.get('uservoice_widget_options', {}))

        identity = get_identity(context, 'uservoice', self._identify)
        if identity:
            identity = IDENTITY % {'options': json.dumps(identity, sort_keys=True)}

        trigger = context.get('uservoice_add_trigger',
                              getattr(settings, 'USERVOICE_ADD_TRIGGER', True))

        html = TRACKING_CODE % {'widget_key': widget_key,
                                'options':  json.dumps(options, sort_keys=True),
                                'trigger': TRIGGER if trigger else '',
                                'identity': identity if identity else ''}
        return html

    def _identify(self, user):
        name = user.get_full_name()
        if not name:
            name = user.username
        return {'name': name, 'email': user.email}


def contribute_to_analytical(add_node):
    UserVoiceNode()  # ensure properly configured
    add_node('body_bottom', UserVoiceNode)
